function varargout = MNP(varargin)
% MNP simulates the optical properties of metallic nanoparticles using
% the Mie theory. MNP is written for didactic purposes for the benefit
% of the NANOLAB-Educational nanoscience project (www.nanolab.unimore.it)
% of the Physics Department of the Univ. of Modena and Reggio E., Italy
%
% MNP calculated the exctincion (absorption+scattering0 spectra of a
% metallic nanoparticle whose complex dielectric function is known, and
% simulates the color of a solution containng the nanoparticles in a
% light transmission experiment.
%
% The user can change the nanoparticle material (Au, Ag, or user defined)
% and size, the solvent dielectric constant, the light source (daylight,
% tungsten lamp, white source or user defined).
%
% Documentation can be found in the accompanying file MNPHelp.pdf.
% MNP excutables and source files can be downloaded at
% www.nanolab.unimore.it
%
% (c) 2012, Guido Goldoni, guido.goldoni@unimore.it
%

% -------------------------------------------- Help generated by MATLAB
% MNP MATLAB code for MNP.fig
%      MNP, by itself, creates a new MNP or raises the existing
%      singleton*.
%
%      H = MNP returns the handle to a new MNP or the handle to
%      the existing singleton*.
%
%      MNP('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in MNP.M with the given input arguments.
%
%      MNP('Property','Value',...) creates a new MNP or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before MNP_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to MNP_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help MNP

% Last Modified by GUIDE v2.5 19-Apr-2012 09:33:46

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @MNP_OpeningFcn, ...
                   'gui_OutputFcn',  @MNP_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before MNP is made visible.
function MNP_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to MNP (see VARARGIN)

global graph_idx;

% Choose default command line output for MNP
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes MNP wait for user response (see UIRESUME)
% uiwait(handles.figure1);

% show the Nanolab Logo
axes(handles.axes18);
logo = imread('LogoNanoLab.jpg');
imagesc(logo)
axis image
axis(gca,'off');

% initilize the plots
graph_idx = 0;

% get default x range
xmin = str2double(get(handles.edit5,'String'));
xmax = str2double(get(handles.edit6,'String'));

% color bars
axis(handles.axes19,'off');
axis(handles.axes20,'off');
axis(handles.axes21,'off');

% top left - spectrum
axes(handles.axes1);
plot([xmin xmax],[0 0],'Color','black');
set(gca,'FontSize',7);
title('Spectrum');
xlim([xmin xmax]); ylim([0. 1.]);
xlabel('Wavelength (nm)'); ylabel('Arbitrary units');

% top right - transimetted light 
axes(handles.TransmittedLight);
plot([xmin xmax],[0 0],'Color','black');
set(gca,'FontSize',7);
title('Transmitted light');
xlim([xmin xmax]); ylim([0. 1.05]);
xlabel('Wavelength (nm)'); ylabel('Arbitrary units');

% bottom left - color matching functions
axes(handles.axes4);
plot([xmin xmax],[0 0],'Color','black');
set(gca,'FontSize',7);
title('Color matching functions');
xlim([xmin xmax]); ylim([0. 1.]);
xlabel('Wavelength (nm)'); ylabel('Arbitrary units');

% bottom right - simulated color
axis(handles.axes2,'off');
axis(handles.axes6,'off');
axis(handles.axes7,'off');
axis(handles.axes8,'off');
axis(handles.axes9,'off');

% bottom right - active dots
axis(handles.axes10,'off');
axis(handles.axes11,'off');
axis(handles.axes12,'off');
axis(handles.axes13,'off');
axis(handles.axes14,'off');


% --- Outputs from this function are returned to the command line.
function varargout = MNP_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in pushbutton1.
function pushbutton1_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global graph_idx;
global material MaterialFile MaterialPath;
global light_source IlluminantFile IlluminantPath;
global wave qext qabs qsca;

% Define constants ...
h = 6.626068e-34;        % Planck's m2kg/s
c = 299792458;           % speed of light in vacuum m/s
eV2Joule = 1.602176e-19; % eV -> Joule
m2nm = 1e9;              % m -> nm

% matrix for colorimetry
m = [0.49 0.31 0.20;
     0.17697 0.81240 0.01063;
     0.00 0.01 0.99] * (1/0.17697);

% Initialize hint
Hint = 'Press button Run simulation';

% Increase graph counter
graph_idx = mod(graph_idx,5)+1;

% Check if the graphs are too crowded
if graph_idx > 4
    Hint = 'To clear the graphs and start over press button Clear graphics.';
end

% Message for the second run
if graph_idx == 2
    Hint = 'The red diamond indicates the current color.';
end

% Get diameter
diameter = str2double(get(handles.edit2,'String'));

% Check if the diameter is too small
if diameter < 5
    Hint = 'Results might be inaccurate for diameters smaller than ~5 nm.';
end

% Get environemnt refractive index
medium_ref_index = str2double(get(handles.edit3,'String'));

% Check refractive index value
if medium_ref_index < 1
    Hint = ['Vacuum refractive index is 1.' ...
            'Smaller values are physically meaningless.'];
end

% Get minimum wavelegth
min_wave = str2double(get(handles.edit5,'String'));

% Check wether minimum wavelength is in the visible range
if min_wave > 380 
	Hint = ['Minimum wavelength does not cover visible range.' ...
            'Color might be inaccurate.'];
end

% Get maximum wavelegth
max_wave = str2double(get(handles.edit6,'String'));

% Check wether minimum wavelength is in the visible range
if max_wave < 825 
    Hint = ['Maximum wavelength does not cover visible range.' ...
            'Color might be inaccurate.'];
end

% Get thickness
thickness = str2double(get(handles.edit8,'String'));

% Get graph type
graph_type = [get(handles.togglebutton1,'Value') == ...
                 get(handles.togglebutton1,'Max'), ...
              get(handles.togglebutton2,'Value') == ...
                 get(handles.togglebutton1,'Max'), ...
              get(handles.togglebutton3,'Value') == ...
                 get(handles.togglebutton1,'Max')];
if graph_type == [1 1 1]
    graph = 'all';
elseif graph_type == [1 1 0]
    graph = 'es';
elseif graph_type == [1 0 1]
    graph = 'ea';
elseif graph_type == [1 0 0]
    graph = 'e';
elseif graph_type == [0 1 1]
    graph = 'sa';
elseif graph_type == [0 0 1]
    graph = 'a';
elseif graph_type == [0 1 0]
    graph = 's';
elseif graph_type == [0 0 0]
    graph = 'a';
end

% Set wavelengths grid spacing to unit in [nm]
dwave = 1;

% Create wavelengths grid
wave = min_wave:dwave:max_wave;

% prepare new spectrum
qext = []; qabs = []; qsca = [];

% read the relevant datasets for the rifraction index
switch material
 case 1
    load GOLD_JC.txt
    tab_energy = GOLD_JC(:,1);
    tab_n = GOLD_JC(:,2);
    tab_k = GOLD_JC(:,3);
  case 2
    load SILVER_JC.txt
    tab_energy = SILVER_JC(:,1);
    tab_n = SILVER_JC(:,2);
    tab_k = SILVER_JC(:,3);
  case 3
    fid = fopen([MaterialPath MaterialFile]);
    A = fscanf(fid,'%g %g %g', [3 inf]);
    fclose(fid);
    tab_energy = A(1,:);
    tab_n = A(2,:);
    tab_k = A(3,:);
    MaterialLabel = sprintf('%s',MaterialFile);
end

% convert tabulated energies from eV to Joule
tab_energy = tab_energy*eV2Joule;

% convert tabulated energies in Joules to wavelengths in nm
tab_wave = m2nm*h*c*(1./tab_energy(:));

% calculate spline interpolant of real and imaginary refractive indeces
pp_n = spline(tab_wave,tab_n);
pp_k = spline(tab_wave,tab_k);

% evaluate complex metal refractive index on the requested grid 
metal_ref_index = ppval(pp_n,wave) + 1i*ppval(pp_k,wave);

% compute the relative refraction index w.r.t. the environment
relative_ref_index = metal_ref_index/medium_ref_index;

% compute size pamaters as wavevector (2 pi/wavelength) * radius
size_parameter = (2*pi*medium_ref_index)*(1./wave(:))*(diameter/2);

% prova plot interpolazione DEBUG
% plot(tab_wave,tab_n,'o',tab_wave,tab_k,'o',...
%      wave, real(metal_ref_index), wave, imag(metal_ref_index));
% xlim([wave(1) wave(length(wave))])

% Calculate exctinction, scattering and absorption spectra
% Use Metzler library v2.
for iw = 1:length(wave)
  q = Mie(relative_ref_index(iw),size_parameter(iw));
  q = q/diameter; % normalize at constant mass density
  qext(iw) = q(1);
  qsca(iw) = q(2);
  qabs(iw) = q(3);
end

% Select the left-top panel
axes(handles.axes1);

% Get graph type
switch graph
    case 'all'
        qmax = max([max(qext) max(qsca) max(qabs)]);
        h = plot(wave,qext/qmax,wave,qsca/qmax,wave,qabs/qmax);
        legend('Extinction','Scattering','Absorption');
    case 'es'
        qmax = max([max(qext) max(qsca)]);
        h = plot(wave,qext/qmax,wave,qsca/qmax);
        legend('Extinction','Scattering');
    case 'ea'
        qmax = max([max(qext) max(qabs)]);
        h = plot(wave,qext/qmax,wave,qabs/qmax);
        legend('Extinction','Absorption');
    case 'e'
        qmax = max(qext);
        h = plot(wave,qext/qmax);
        legend('Extinction');
    case 'sa'
        qmax = max([max(qsca) max(qabs)]);
        h = plot(wave,qsca/qmax,wave,qabs/qmax);
        legend('Scattering','Absorption');
    case 'a'
        qmax = max(qabs);
        h = plot(wave,qabs/qmax);
        legend('Absorption');
    case 's'
        qmax = max(qsca);
        h = plot(wave,qsca/qmax);
        legend('Scattering');
end

% Set line width
set(h,'LineWidth',1.0);

% Set graph limits
xlim([min_wave max_wave]);
ylim([0 1.05]);

% Set graphs labels
title('Spectrum');
xlabel('Wavelength (nm)');
ylabel('Arbitrary units');

% Next plot will be added
hold on;

% top left - color bar
axes(handles.axes19);
[lambda, RGB] = createSpectrum('1964_full');
h = image(lambda, 1:size(RGB,1), RGB);
xlim([min_wave max_wave]); ylim([0. 1.]);
set(h, 'AlphaData', 0.8);
set(gca,'XLim',[min_wave max_wave]);
axis(gca,'off');

% Get illuminat spectrum
switch light_source
    case 1
        [lambda_source, energy] = illuminant('d65'); % sun
        label = 'Daylight (d65)';
    case 2
        [lambda_source, energy] = illuminant('a');   % lamp
        label = 'Tungsten filament';
    case 3
        [lambda_source, energy] = illuminant('ee');  % white light
        label = 'White spectrum';
    case 4
        fid = fopen([IlluminantPath IlluminantFile]);
        A = fscanf(fid,'%g %g', [2 inf]);
        fclose(fid);
        lambda_source = A(1,:);
        energy = A(2,:);
        label = sprintf('%s',IlluminantFile);
end

% Get color matching functions
[lambda_fcn, xFcn, yFcn, zFcn] = colorMatchFcn('1964_FULL');

% calculate the interpolant for the source energy distribution
pp_en = spline(lambda_source,energy);

% calculate the interpolant for the color matching functions
pp_xf = spline(lambda_fcn,xFcn);
pp_yf = spline(lambda_fcn,yFcn);
pp_zf = spline(lambda_fcn,zFcn);

% Put source energy on the same grid of absorption
en = ppval(pp_en,wave);

% Put color mathcing functions on the same grid of absorption
xf = ppval(pp_xf,wave);
yf = ppval(pp_yf,wave);
zf = ppval(pp_zf,wave);

% normalized absorption intensity
k = qabs/max(qabs);
%k = qext/max(qext);

% normalized source intensity
en = en/max(en);

% spectral distribution 
%spec_distr = en.*exp(-k.*thickness)';

spec_distr = en.*exp(-k.*thickness);

% Plot the transimetted light 
axes(handles.TransmittedLight);
h = plot(wave,en,wave,spec_distr);
xlim([min_wave max_wave]);
ylim([0 1.05]);
title('Transmitted light');
xlabel('Wavelength (nm)');
ylabel('Arbitrary units');
legend(label,'Transmitted');
set(h,'LineWidth',1.0);

% color bar
axes(handles.axes20);
[lambda, RGB] = createSpectrum('1964_full');
h = image(lambda, 1:size(RGB,1), RGB);
xlim([min_wave max_wave]); ylim([0. 1.]);
set(h, 'AlphaData', 0.8);
set(gca,'XLim',[min_wave max_wave]);
axis(gca,'off');


% Plot the color matching functions
axes(handles.axes4);
h = plot(wave,xf,wave,yf,wave,zf);
xlim([min_wave max_wave]);
ylim([0 2.2]);
title('Color matching functions');
xlabel('Wavelength (nm)');
ylabel('Arbitrary units');
legend('x','y','z');
set(h,'LineWidth',1.0);

% color bar
axes(handles.axes21);
[lambda, RGB] = createSpectrum('1964_full');
h = image(lambda, 1:size(RGB,1), RGB);
xlim([min_wave max_wave]); ylim([0. 1.]);
set(h, 'AlphaData', 0.8);
set(gca,'XLim',[min_wave max_wave]);
axis(gca,'off');


% Compute projections
xyz = [trapz(xf.*spec_distr), ...
       trapz(yf.*spec_distr), ...
       trapz(zf.*spec_distr)];
%xyz = xyz/sum(xyz);

% transform to RGB
rgb = inv(m)*xyz';
rgb = rgb/max(rgb); % ?????????

% find wavelength of maximum absorption
[M, Indices] = max(qabs);
wave_max = wave(max(Indices));

% Plot the perceived color and output max wavelength
switch graph_idx
    case 1
        axes(handles.axes2);
        filledCircle([0 0],1,500,rgb'); axis off;
        set(handles.text21,'String',num2str(wave_max))
        axes(handles.axes14);
        cla; axis off;
        axes(handles.axes10);
        filledCircle([0 0],0.1,5,'red'); axis off;
    case 2
        axes(handles.axes6);
        filledCircle([0 0],1,500,rgb'); axis off;
        set(handles.text24,'String',num2str(wave_max))
        axes(handles.axes10);
        cla; axis off;
        axes(handles.axes11);
        filledCircle([0 0],0.1,5,'red'); axis off;
    case 3
        axes(handles.axes7);
        filledCircle([0 0],1,500,rgb'); axis off;
        set(handles.text25,'String',num2str(wave_max))
        axes(handles.axes11);
        cla; axis off;
        axes(handles.axes12);
        filledCircle([0 0],0.1,5,'red'); axis off;
    case 4
        axes(handles.axes8);
        filledCircle([0 0],1,500,rgb'); axis off;
        set(handles.text26,'String',num2str(wave_max))
        axes(handles.axes12);
        cla; axis off;
        axes(handles.axes13);
        filledCircle([0 0],0.1,5,'red'); axis off;
    case 5
        axes(handles.axes9);
        filledCircle([0 0],1,500,rgb'); axis off;
        set(handles.text27,'String',num2str(wave_max))
        axes(handles.axes13);
        cla; axis off;
        axes(handles.axes14);
        filledCircle([0 0],0.1,5,'red'); axis off;
end

% output hints
set(handles.text33,'String',Hint)

% everything done!



% --- Executes during object creation, after setting all properties.
function edit2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function popupmenu1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

global material;

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Get material type
material = get(hObject,'Value');


function h = filledCircle(center,r,N,color)
%---------------------------------------------------------------------------------------------
% FILLEDCIRCLE Filled circle drawing
% 
% filledCircle(CENTER,R,N,COLOR) draws a circle filled with COLOR that 
% has CENTER as its center and R as its radius, by using N points on the 
% periphery.
%
% Usage Examples,
%
% filledCircle([1,3],3,1000,'b'); 
% filledCircle([2,4],2,1000,'r');
%
% Sadik Hava <sadik.hava@gmail.com>
% May, 2010
%
% Inspired by: circle.m [Author: Zhenhai Wang]
%---------------------------------------------------------------------------------------------

THETA=linspace(0,2*pi,N);
RHO=ones(1,N)*r;
[X,Y] = pol2cart(THETA,RHO);
X=X+center(1);
Y=Y+center(2);
h=fill(X,Y,color);
set(h,'EdgeColor','None');
axis square;


% COPYRIGHT STUFF... :D (Since I am modifying Zhenhai's code.)
%
% Copyright (c) 2002, Zhenhai Wang
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.




% --- Executes during object creation, after setting all properties.
function edit3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




% --- Executes during object creation, after setting all properties.
function edit4_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




% --- Executes during object creation, after setting all properties.
function edit5_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




% --- Executes during object creation, after setting all properties.
function edit6_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




% --- Executes during object creation, after setting all properties.
function edit7_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit7 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function slider1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes during object creation, after setting all properties.
function popupmenu3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

global light_source;

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Get illuminant type
light_source = get(hObject,'Value');


% --- Executes during object creation, after setting all properties.
function edit8_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit8 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pushbutton2.
function pushbutton2_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global graph_idx;

% clear simulated colors
axes(handles.axes2); cla;
axes(handles.axes6); cla;
axes(handles.axes7); cla;
axes(handles.axes8); cla;
axes(handles.axes9); cla;

% clear active dot
axes(handles.axes10); cla; axis off;
axes(handles.axes11); cla; axis off;
axes(handles.axes12); cla; axis off;
axes(handles.axes13); cla; axis off;
axes(handles.axes14); cla; axis off;

% clear max wavelength
set(handles.text21,'String',' ')
set(handles.text24,'String',' ')
set(handles.text25,'String',' ')
set(handles.text26,'String',' ')
set(handles.text27,'String',' ')

% clear plots
axes(handles.axes1); legend('off'); cla;
axes(handles.TransmittedLight); legend('off'); cla;
axes(handles.axes4); legend('off'); cla;

% clear plot counter
graph_idx = 0;


% --- Executes on button press in pushbutton3.
function pushbutton3_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% open the 'save file' dialog box and get file name
[FileName,PathName] = uiputfile({'*.jpg';'*.png';'*.pdf'; ...
                                 '*.tif';'*.eps';'*.bmp'},...
                        'Save window image as ...','MNP.jpg');

% compose the filename with path
FigFileName = [PathName FileName];

% export image
export_fig(sprintf('%s',FigFileName));


function edit2_Callback(hObject, eventdata, handles)
% hObject    handle to edit2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit2 as text
%        str2double(get(hObject,'String')) returns contents of edit2 as a double



function edit5_Callback(hObject, eventdata, handles)
% hObject    handle to edit5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit5 as text
%        str2double(get(hObject,'String')) returns contents of edit5 as a double



function edit6_Callback(hObject, eventdata, handles)
% hObject    handle to edit6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit6 as text
%        str2double(get(hObject,'String')) returns contents of edit6 as a double


% --- Executes on selection change in popupmenu3.
function popupmenu3_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global light_source IlluminantFile IlluminantPath;

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu3 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu3

% Get illuminant type
light_source = get(hObject,'Value');

% Get file name for the illuminant
if light_source==4
  [IlluminantFile,IlluminantPath] = uigetfile({'*.dat';'*.txt'}, ...
        'Load illuminant from ...');
end

function edit8_Callback(hObject, eventdata, handles)
% hObject    handle to edit8 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit8 as text
%        str2double(get(hObject,'String')) returns contents of edit8 as a double


% --- Executes on button press in togglebutton3.
function togglebutton3_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of togglebutton3

function edit3_Callback(hObject, eventdata, handles)
% hObject    handle to edit3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit3 as text
%        str2double(get(hObject,'String')) returns contents of edit3 as a double


% --- Executes on selection change in popupmenu1.
function popupmenu1_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu1 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu1

global material MaterialFile MaterialPath;

% Get material type
material = get(hObject,'Value');

% Get file name for the material dielectric function
if material==3
  [MaterialFile,MaterialPath] = uigetfile({'*.dat';'*.txt'}, ...
        'Load dielectric function from ...');
end

% --- Executes during object creation, after setting all properties.
function axes18_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes18 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes18


% --- Executes on button press in pushbutton4.
function pushbutton4_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes during object creation, after setting all properties.
function axes2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes2


% --- Executes on button press in pushbutton6.
function pushbutton6_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Open a new window
%MNPHelp
winopen('MNPHelp.pdf')

% --------------------------------------------------------------------
function Untitled_1_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes during object creation, after setting all properties.
function text33_CreateFcn(hObject, eventdata, handles)
% hObject    handle to text33 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes on button press in pushbutton7.
function pushbutton7_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton7 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global wave qext qabs qsca;

% open the 'save file' dialog box and get file name and path
[FileName,PathName] = uiputfile('MNP.dat','Save spectrum as ...');

% compose file name with full path
DataFileName = [PathName FileName];

% open file
DataFileID = fopen(sprintf('%s',DataFileName),'wt');

% export data to the specified file
data = [wave;qext;qabs;qsca];
fprintf(DataFileID,'%f %f %f %f\n',data);

% close file
fclose(DataFileID);


% --- Executes during object creation, after setting all properties.
function axes19_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes19 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes19


% --- Executes during object creation, after setting all properties.
function axes20_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes20 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes20


% --- Executes during object creation, after setting all properties.
% hObject    handle to axes21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes21


% --- Executes during object creation, after setting all properties.
function axes21_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes21


% --- Executes when figure1 is resized.
function figure1_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
